const host = ''; //Add server host
const email = ''; // Add email
const password = ''; // Add password

const entities = require('@jetbrains/youtrack-scripting-api/entities');
const http = require('@jetbrains/youtrack-scripting-api/http');

let refreshToken = null;

function getAuthToken() {
  if (refreshToken) {
    return Promise.resolve(refreshToken);
  }

  const connection = new http.Connection(host);
  const auth = {
    email: email,
    password: password
  };

  connection.addHeader('Content-Type', 'application/json');
  const result = connection.postSync('/api/auth/login', [], auth);
  if (result && result.code === 200) {
    const body = JSON.parse(result.body);
    refreshToken = 'Bearer ' + body.tokens.refreshToken;
    return refreshToken;
  } else {
    throw new Error('Authentication failed');
  }
}

exports.rule = entities.Issue.onChange({
  title: 'Send HTTP Request on Issue Change',
   guard: (ctx) => {
    const issue = ctx.issue;
    
    const hasPriority = issue.Priority !== undefined;
    const hasState = issue.State !== undefined;
    
    return (hasPriority && issue.isChanged('priority')) || 
           (hasState && issue.isChanged('state')) ||
           issue.isChanged('summary') ||
           issue.isChanged('description');
  },
  action: async(ctx) => {
    const issue = ctx.issue;

    try {
      const token = await getAuthToken();
      const connection = new http.Connection(host);

      const payload = {
        type: 'youtrack',
        issueId: issue.id,
        priority: issue.Priority ? issue.Priority.name : null,
        state: issue.State ? issue.State.name : null,
        url: issue.url,
        description: issue.description,
        summary: issue.summary,
        updated: new Date().toISOString()
      };

     
      connection.addHeader('Content-Type', 'application/json');
      connection.addHeader('Authorization', token);
      
      const response = await connection.postSync('/api/integrations/youtrack/webhook', [], payload);
      if (response && response.code === 200) {
        console.log('Successfully sent data for issue: '+ issue.id);
      } else {
        console.error('Failed to send data for issue: ' + issue.id);
      }
    } catch (error) {
      console.error('Error:', error.message);
    }
  }
});